#ifndef GDS_MATH_ARCH_HH
#define GDS_MATH_ARCH_HH

/*  Test the compiler/platform architecture to see if it supports 
 *  vectorized calculations using the SSE2-4 and/or avx instruction
 *  sets.
 */

//==================================  Figure out the platform
#if defined(__GNUG__) && (defined(__i386__) || defined(__x86_64__))
  #if defined(__AVX__)
    #define GDS_USE_AVX 1
    #define GDS_USE_SSE4 1
    #define GDS_USE_SSE4_2 1
  #elif defined(__SSE4_2__)
    #define GDS_USE_SSE4_2 1
    #define GDS_USE_SSE4 1
  #elif defined(__SSE4_1__)
    #define GDS_USE_SSE4 1
  #elif defined(__SSSE3__)
    #define GDS_USE_SSE3 1
  #elif defined(__SSE2__)
    #define GDS_USE_SSE2 1
  #else
    #warning "SSE2 not enabled"
    #define GDS_USE_BASE 1
  #endif
#else 
  #warning "Not a gnu compilation"
  #define GDS_USE_BASE 1
#endif // end test of i386/gcc

//==================================  Useful data types
#if defined(GDS_USE_SSE2) || defined(GDS_USE_SSE3) || defined(GDS_USE_SSE4)
#define GDS_VEC_LENGTH 16
typedef double vecd128 __attribute__ ((vector_size(GDS_VEC_LENGTH)));
typedef float  vecf128 __attribute__ ((vector_size(GDS_VEC_LENGTH)));
typedef int    veci128 __attribute__ ((vector_size(GDS_VEC_LENGTH)));
/*---- Mask to test pointer alignment  */
#define VEC128_ADMASK (sizeof(vecd128)-1)
union vd2map {
  vecd128 v;
  double d[2];
};
#endif // end test of sse2

#ifdef GDS_USE_AVX
#undef GDS_VEC_LENGTH
#define GDS_VEC_LENGTH 32
typedef double vecd256 __attribute__ ((vector_size(GDS_VEC_LENGTH)));
typedef float  vecf256 __attribute__ ((vector_size(GDS_VEC_LENGTH)));
/*---- Mask to test pointer alignment  */
#define VEC256_ADMASK (sizeof(vecd256)-1)
union vd4map {
  vecd256 v;
  double d[4];
};
#endif // end test of avx

#endif // !defined(gds_math_arch_hh)
